# Supports Organizer.rb
=begin rdoc

= dropGC.rb
Copyright 2012-2014 by Rick Wilson - All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

== Information
Author:: Rick Wilson
Organization:: Smustard
Name:: dropGC.rb
Version:: 1.0.3
SU Version:: 4.0
Date:: 2014-06-20
Description:: drop selected groups/components to the first face below each instance

Usage::
* 1:: Plugins>DropGC or right-click>DropGC or SmustardToolbar>DropGC activates the tool

History::
* 1.0.0:: 2012-02-20
	* first version
* 1.0.1::	2014-05-02
	* updated the SmustardToolbar reference from a global to a Smustard module constant
* 1.0.2:: 2014-05-16
	* version 1.0.1 works in SU 2014, but a code block didn't work in earlier versions (due to changes in Ruby).  Fixed this to work in all versions.
* 1.0.3:: 2014-06-20
	* fixed a typo in my previous fix
* 1.0.4:: 2014-10-15
	* changed .count to .length for compatibility with earlier versions of SketchUp (pre-2013)


ToDoList::
	* add options & settings for: show z offset from face below (or from xy plane if no face below)
	* add options & settings for: use hidden geometry (SU8m1 and up)
	* add options & settings for: drop by set amount
	* add options & settings for: drop to Z
	* add options & settings for: drop to selected
	* add options & settings for: drop to selected (virtual extended plane)
	* add options & settings for: drop to intersection plus additional offset up or down
	* add options & settings for: keep G/C above face below
	* add options & settings for: match G/C XY plane to intersected plane below

=end


require "sketchup.rb"

module Smustard
	
	if ($submenu) # legacy programs creating a global
		Smustard::Submenu = $submenu unless Smustard.constants.include?("标高放置")
	else
		Smustard::Submenu = nil unless Smustard.constants.include?("标高放置")
	end
	

	if ($smustard_toolbar)	# legacy programs creating a global
		Smustard::Toolbar = $smustard_toolbar unless Smustard.constants.include?("Toolbar")
			
	else
		Smustard::Toolbar = UI::Toolbar.new("标高放置") unless Smustard.constants.include?("Toolbar")
			
	end

class DropGC
	
	@@menucreated = false
	@@instance ||= self.new;
	
	begin
		@@menunames = IO.readlines(File.dirname(__FILE__)+"/menus/"+File.basename(__FILE__, ".rb")+".menu").collect{|x| x.chop}
	
	rescue
		@@menunames = ["标高放置","标高放置 设置"]
	
	end

	@@cmd_dropgc = UI::Command.new(@@menunames[0]){ @@instance.drop() }
	@@cmd_dropgc.tooltip = "将组或组件放置到下面的几何图形上"
	@@cmd_dropgc.large_icon = "DropGClarge.png"
	@@cmd_dropgc.small_icon = "DropGCsmall.png"
	
	attr_accessor :wysiwig, :last_time, :instance, :selection
	
	def initialize()
		
		@model = Sketchup.active_model
		@selection = @model.selection
		@wysiwig = true
		begin
			Sketchup.active_model.raytest([[0,0,0],[0,0,-1]],@wysiwig)
		rescue
			@wysiwig = "不可用"
		end
		@last_time = 0
		@instance = @@instance
	end
	
	
	def DropGC.instance()
		return @@instance
	end
	
	
	def reinit(wysiwig=true)
		
		@model = Sketchup.active_model
		@selection = @model.selection
		@wysiwig = wysiwig
		
	end
	
	
	def drop()
		@model = Sketchup.active_model
		sel = @model.selection.grep(Sketchup::ComponentInstance) + @model.selection.grep(Sketchup::Group)
		@wysiwig = true
		begin
			Sketchup.active_model.raytest([[0,0,0],[0,0,-1]],@wysiwig)
		rescue
			@wysiwig = "不可用"
		end
		puts "启动 DropGC: wysiwig = #{@wysiwig}"
		(puts "未选择任何内容";return nil) unless sel.kind_of?(Array) || sel.kind_of?(Sketchup::Selection)
		
		down = [0,0,-1]
		qty = sel.length	# 1.0.4
		return nil if qty < 1
		dropped = 0
		
		(Sketchup.version.split('.')[0].to_i)>6 ? @model.start_operation("标高放置",true) : @model.start_operation("标高放置")
		
		start_time = Time.now()
		counting = false
		counting = true if sel.length > 10
		
		sel.each_with_index do |instance,i|
			
			(Sketchup.set_status_text("处理中: #{((i+1)*100)/qty}%") if  (i+1).modulo(qty/10)==0) if (counting && qty>0)
			
			origin = instance.transformation.origin
			
			v,m,b = Sketchup.version.split('.')
			
			if @wysiwig != "不可用" #v.to_i > 8 || (v.to_i == 8 && b.to_i > 4809)
				next unless raytest_result = @model.raytest([origin,down],wysiwig)
			else
				next unless raytest_result = @model.raytest([origin,down])
			end
			
			instance.transform!(Geom::Transformation.translation(raytest_result[0]-origin))
			dropped += 1
			
		end
		
		end_time = Time.now()
		
		@last_time = end_time - start_time
		
		@model.commit_operation()
		
		Sketchup.set_status_text("在 #{dropped} 秒内丢弃了 #{@last_time} 个项目")
		puts "在 #{dropped} 秒内丢弃了 #{@last_time} 个项目"
		
	end
	
	
	def settings()
		#	This code will allow the user to change the DropGC settings.  Settings could include:
		#		* use hidden geometry (SketchUp 8m1 and higher)
		#		* drop by set amount
		#		* drop to Z
		#		* drop to selected
		#		* drop to selected (virtual extended plane)
		#		* drop to intersection plus additional offset up or down
		#		* keep G/C above face below
		#		* match G/C XY plane to intersected plane below
	end
	
	
	def DropGC.create_menu()
		
		unless @@menucreated
		
			Smustard::Toolbar.add_item(@@cmd_dropgc)
			
			Smustard::Submenu ? (menu = Smustard::Submenu) : (menu = UI.menu("Plugins"))
			menu.add_item(@@cmd_dropgc)
			#organizer.add_item(menunames[1]) { cmd_dropgcsettings }
			
			UI.add_context_menu_handler do |menu|
				menu.add_item(@@cmd_dropgc)
			end
		
			@@menucreated = true
		
		end
		
		
	end
	
	
	def DropGC.wysiwig
		puts wysiwig
	end
	
	
end #class DropGC


end #module Smustard

unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
	Smustard::DropGC.create_menu()
end